let HttpUtils = require('./httpUtils.js')
let httpUtils = new HttpUtils()

let constant = require('./constant.js')
const hubAddress = constant.CONTROL_HUB_ADDRESS
const userName = constant.USER_NAME
const password = constant.USER_PASSWORD

const getStatusURL = `http://${hubAddress}/api/system/status`

/**
 * 调用登录接口（/api/login），获取 login cookie。
 * Use '/api/login' interface to log in. Obtain the Login cookie after logging in successfully.
 */
httpUtils.reqStartLog(1, 'login request')
httpUtils.post(`http://${hubAddress}/api/login`,
    {
        params: { id: userName, pass: password },
        isJson: true
    })
    .then((response) => {
        httpUtils.log('Response', response)
        const resCookies = response['headers']['set-cookie']
        if (resCookies && resCookies.length > 0) {
            httpUtils.log('get login cookie', resCookies)
            // set request parameters
            let reqOpts = {
                headers: {
                    'Cookie': resCookies
                }
            }
            // get status
            getStatus(reqOpts)
        } else {
            httpUtils.log('get cookie failed', resCookies)
        }

    })
    .catch((err) => {
        httpUtils.log('Exception', err)
    })

/**
 * 调用获取状态信息接口（get-status），获取 Control Hub 基本信息
 * Use the 'get-status' interface to obtain status and parameters of the Control Hub。
 */
const getStatus = function (reqOpts) {
    httpUtils.reqStartLog(2, 'get-status request')
    httpUtils.get(getStatusURL, reqOpts)
        .then((res) => {
            const data = res.data
            // httpUtils.log('Response', JSON.parse(data))
            httpUtils.log('Response', data)
        })
        .catch((err) => {
            httpUtils.log('Exception', err)
        })
}
