const http = require('http')
const { URL } = require('url')
const fs = require('fs')
const path = require('path')
/**
 * 这个库可以把对象序列化。
 * This library is used to serialize objects.
 */
const querystring = require('querystring')

function HttpUtils () {}

// post
HttpUtils.prototype.post = function (url, options = {}) {
    return new Promise((resolve, reject) => {
        let postData
        if (options.params) {
            if (options.isJson) {
                postData = JSON.stringify(options.params)
            } else {
                postData = querystring.stringify(options.params)
                console.log(postData)
            }
        }
        const urlInfo = new URL(url)
        let opts = {}
        opts.hostname = urlInfo.hostname
        opts.path = urlInfo.pathname + urlInfo.search
        opts.port = urlInfo.port || 80
        opts.timeout = options.timeout || 5000
        opts.method = 'POST'
        opts.headers = {
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'zh-CN,zh;q=0.8,en;q=0.6',
            'Connection': 'keep-alive',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            ...options.headers
        }
        const req = http.request(opts, function (res) {
            const { statusCode, headers } = res
            let error = null
            if (statusCode !== 200) {
                error = new Error('Request Failed.\n' +
                    `Status Code:  ${statusCode}`)
            }
            if (error) {
                res.resume()
                reject(error)
            }

            res.setEncoding('utf8')

            let rawData = ''
            res.on('data', (chunk) => {
                rawData += chunk
            })
            res.on('end', () => {
                const resData = {
                    statusCode,
                    headers,
                    data: headers['content-type'] === 'application/json' ? JSON.parse(rawData) : rawData
                }
                resolve(resData)
            })

        })
        //res是请求后的回调函数拿到的回复,req是请求
        req.on('error', function (e) {
            reject(e.message)
        })

        req.write(postData)
        req.end()
    })
}


// get
HttpUtils.prototype.get = function (url, options = {}) {
    return new Promise((resolve, reject) => {
        const urlInfo = new URL(url)
        let opts = {}
        opts.hostname = urlInfo.hostname
        opts.path = urlInfo.pathname + urlInfo.search
        opts.port = urlInfo.port || 80
        opts.headers = options.headers || {}
        opts.timeout = options.timeout || 5000

        // set timeout, default 5s
        const requestTimerID = setTimeout(() => {
            httpGet.abort()
        }, opts.timeout)

        let httpGet = http.get(opts, (res) => {
            clearTimeout(requestTimerID)
            const { statusCode, headers } = res
            let error = null
            if (statusCode !== 200) {
                error = new Error('Request Failed.\n' +
                    `Status Code:  ${statusCode}`)
            }
            if (error) {
                res.resume()
                reject(error)
            }

            res.setEncoding('utf8')
            let rawData = ''
            res.on('data', (chunk) => {
                rawData += chunk
            })
            res.on('end', () => {
                const resData = {
                    statusCode,
                    headers,
                    data: headers['content-type'] === 'application/json' ? JSON.parse(rawData) : rawData
                }
                resolve(resData)
            })
        }).on('error', (e) => {
            reject(e.message)
        })
    })
}

// upload
HttpUtils.prototype.upload = function (url, filePath, options = {}) {

    return new Promise((resolve, reject) => {
        const urlInfo = new URL(url)

        let boundaryKey = '----WebKitFormBoundarypQg2o6On0TJpWqIk'
        let opts = {
            hostname: urlInfo.hostname,
            path: urlInfo.pathname + urlInfo.search,
            port: urlInfo.port || 80,
            method: 'POST',
            headers: {
                'Content-Type': 'multipart/form-data; boundary=' + boundaryKey,
                'Cookie': options.headers.Cookie
            }
        }

        //read file
        fs.readFile(filePath, function (err, data) {

            if (err) {
                reject(err)
                return false
            }

            //"multipart/form-data" encoding, see https://www.w3.org/TR/html4/interact/forms.html#didx-multipartform-data
            let payloadStart = `--${boundaryKey}\r\n`
            payloadStart += `Content-Disposition: form-data; name="file"; filename="${path.basename(filePath)}"\r\n`
            payloadStart += 'Content-Type: application/octet-stream\r\n\r\n'

            let payloadEnd = '\r\n--'
            payloadEnd += boundaryKey + '--\r\n'

            // to Buffer
            const payloadBufStart = Buffer.from(payloadStart)
            const payloadBufEnd = Buffer.from(payloadEnd)

            // payloadBufStart + data + payloadBufEnd
            const payloadBufs = [payloadBufStart, data, payloadBufEnd]
            const payloadBuf = Buffer.concat(payloadBufs)

            const payloadLength = Buffer.byteLength(payloadBuf)

            const req = http.request(opts, (res) => {
                let resData = ''
                res.setEncoding('utf8')
                res.on('data', (chunk) => {
                    resData += chunk
                })
                res.on('end', () => {
                    try {
                        resolve(JSON.parse(resData))
                    } catch (err) {
                        reject(resData)
                    }

                })
            })

            req.on('error', (e) => {
                reject(e.message)
            })

            req.setHeader('Content-Length', payloadLength)
            req.write(payloadBuf)
            req.end()
        })
    })
}


HttpUtils.prototype.reqStartLog = function (step, msg, pos = '') {
    console.info(`\n==> ${step}. ${msg}    ${pos}`)
}


HttpUtils.prototype.log = function (tag, msg) {
    console.info(`\n>>>>>>>> ${tag}:`)
    if (msg) {
        console.info(msg)
    }
}

module.exports = HttpUtils
